@@ -602,10 +602,10 @@ usage(int exitval, FILE *fp)
 	fputs(_("\t-i includefile\t\t--include=includefile\n"), fp);
 	fputs(_("\t-l library\t\t--load=library\n"), fp);
 	/*
-	 * TRANSLATORS: the "fatal" and "invalid" here are literal
+	 * TRANSLATORS: the "fatal", "invalid" and "no-ext" here are literal
 	 * values, they should not be translated. Thanks.
 	 */
-	fputs(_("\t-L[fatal|invalid]\t--lint[=fatal|invalid]\n"), fp);
+	fputs(_("\t-L[fatal|invalid|no-ext]\t--lint[=fatal|invalid|no-ext]\n"), fp);
 	fputs(_("\t-M\t\t\t--bignum\n"), fp);
 	fputs(_("\t-N\t\t\t--use-lc-numeric\n"), fp);
 	fputs(_("\t-n\t\t\t--non-decimal-data\n"), fp);
@@ -749,6 +749,7 @@ init_args(int argc0, int argc, const char *argv0, char **argv)
 {
 	int i, j;
 	NODE *sub, *val;
+	NODE *shadow_node = NULL;
 
 	ARGV_node = install_symbol(estrdup("ARGV", 4), Node_var_array);
 	sub = make_number(0.0);
@@ -756,15 +757,32 @@ init_args(int argc0, int argc, const char *argv0, char **argv)
 	val->flags |= USER_INPUT;
 	assoc_set(ARGV_node, sub, val);
 
+	if (do_sandbox) {
+		shadow_node = make_array();
+		sub = make_string(argv0, strlen(argv0));
+		val = make_number(0.0);
+		assoc_set(shadow_node, sub, val);
+	}
+
+
 	for (i = argc0, j = 1; i < argc; i++, j++) {
 		sub = make_number((AWKNUM) j);
 		val = make_string(argv[i], strlen(argv[i]));
 		val->flags |= USER_INPUT;
 		assoc_set(ARGV_node, sub, val);
+
+		if (do_sandbox) {
+			sub = make_string(argv[i], strlen(argv[i]));
+			val = make_number(0.0);
+			assoc_set(shadow_node, sub, val);
+		}
 	}
 
 	ARGC_node = install_symbol(estrdup("ARGC", 4), Node_var);
 	ARGC_node->var_value = make_number((AWKNUM) j);
+
+	if (do_sandbox)
+		init_argv_array(ARGV_node, shadow_node);
 }
 
 
@@ -1262,6 +1280,9 @@ catchsig(int sig)
 	        || sig == SIGBUS
 #endif
 	) {
+		if (errcount > 0)	// assume a syntax error corrupted our data structures
+			exit(EXIT_FATAL);
+
 		set_loc(__FILE__, __LINE__);
 		msg(_("fatal error: internal error"));
 		/* fatal won't abort() if not compiled for debugging */
@@ -1279,6 +1300,9 @@ catchsig(int sig)
 static int
 catchsegv(void *fault_address, int serious)
 {
+	if (errcount > 0)	// assume a syntax error corrupted our data structures
+		exit(EXIT_FATAL);
+
 	set_loc(__FILE__, __LINE__);
 	msg(_("fatal error: internal error: segfault"));
 	fflush(NULL);
@@ -1598,7 +1622,7 @@ parse_args(int argc, char **argv)
 
 #ifndef NO_LINT
 		case 'L':
-			do_flags |= DO_LINT_ALL;
+			do_flags |= (DO_LINT_ALL|DO_LINT_EXTENSIONS);
 			if (optarg != NULL) {
 				if (strcmp(optarg, "fatal") == 0)
 					lintfunc = r_fatal;
@@ -1606,6 +1630,9 @@ parse_args(int argc, char **argv)
 					do_flags &= ~DO_LINT_ALL;
 					do_flags |= DO_LINT_INVALID;
 				}
+				else if (strcmp(optarg, "no-ext") == 0) {
+					do_flags &= ~DO_LINT_EXTENSIONS;
+				}
 			}
 			break;
 
